/*
    Razkazhi za men na dushterya ti
    Kogato v skuta ti zaradi muzh zaplache
*/

#include <algorithm>
#include <iostream>
#include <numeric>
#include <cassert>
#include <vector>

typedef long long llong;
const int MAXN = 16000 + 10;
const int MAXLOG = 14;
const int INF  = 1e9;

int parent[MAXN];
int depth[MAXN];
int back[MAXN];
bool vis[MAXN];
std::vector <int> g[MAXN];

void dfs(int node, int par)
{
    vis[node] = true;
    parent[node] = par;
    depth[node] = (node == 0 ? 0 : depth[par] + 1);
    for (const int &u : g[node])
    {
        if (par == u)
        {
            continue;
        }

        if (vis[u])
        {
            if (depth[u] > depth[node])
            {
                continue;
            }
            
            assert(back[node] == 0);
            back[node] = depth[node] - depth[u];
            continue;
        }

        dfs(u, node);
    }
}

std::vector <bool> sol;

void push(std::vector <bool> &to, int num)
{
    for (int lg = MAXLOG - 1 ; lg >= 0 ; --lg)
    {
        to.push_back(num & (1 << lg));
    }
}

int n;
bool skip[MAXN];
std::vector <bool> encode(int N, std::vector <std::pair <int,int>> edges)
{
    n = N;
    sol.clear();
    for (int i = 0 ; i < n ; ++i)
    {
        parent[i] = depth[i] = back[i] = vis[i] = 0;
        g[i].clear();
    }

    for (int i = 0 ; i < edges.size() ; ++i)
    {
        g[edges[i].first].push_back(edges[i].second);
        g[edges[i].second].push_back(edges[i].first);
    }

    dfs(0, -1);
    for (int i = 1 ; i < n ; ++i)
    {
        push(sol, parent[i]);
    }

    std::vector <int> perm(n);
    std::iota(perm.begin(), perm.end(), 0);
    std::sort(perm.begin(), perm.end(), [&](int x, int y)
    {
        return depth[x] > depth[y] || (depth[x] == depth[y] && x > y);
    });

    std::fill(skip + 1, skip + n, false);
    skip[0] = true;

    for (int i = 0 ; i < n ; ++i)
    {
        int node = perm[i];
        if (skip[node])
        {
            continue;
        }

        if (back[node] == 0)
        {
            sol.push_back(0);
            continue;
        }

        int next = node;
        for (int j = 0 ; j + 1 < back[node] ; ++j)
        {
            sol.push_back(1);
            next = parent[next];
            skip[next] = true;
        }
        
        sol.push_back(0);
    }
    
    
    return sol;
}

void dfsDecode(int node)
{
    for (const int &u : g[node])
    {
        depth[u] = depth[node] + 1;
        dfsDecode(u);
    }
}


std::vector <std::pair <int,int>> decode(int N, std::vector <bool> data)
{
    n = N;
    int ptr = 0;
    parent[0] = -1;
    std::vector <std::pair <int,int>> answer;
    for (int i = 1 ; i < n ; ++i)
    {
        int p = 0;
        for (int j = 0 ; j < MAXLOG ; ++j)
        {
            p <<= 1;
            p += data[ptr++];
        }
        
        parent[i] = p;
        answer.push_back({p, i});
    }
    
    for (int i = 0 ; i < n ; ++i)
    {
        g[i].clear();
        depth[i] = 0;
    }
    
    for (int i = 1 ; i < n ; ++i)
    {
        g[parent[i]].push_back(i);
    }
    
    dfsDecode(0);
    std::vector <int> perm(n);
    std::iota(perm.begin(), perm.end(), 0);
    std::sort(perm.begin(), perm.end(), [&](int x, int y)
    {
        return depth[x] > depth[y] || (depth[x] == depth[y] && x > y);
    });
    
    std::fill(skip + 1, skip + n, false);
    skip[0] = true;
    
    for (int i = 0 ; i < n ; ++i)
    {
        int node = perm[i];
        if (skip[node])
        {
            continue;
        }
        
        if (data[ptr] == 0)
        {
            ptr++;
            continue;
        }
        
        int next = parent[node];
        while (data[ptr] == 1)
        {
            skip[next] = true;
            next = parent[next];
            ptr++;
        }
        
        answer.push_back({node, next});
        ptr++;
    }
    
    return answer;
}