/*
    Razkazhi za men na dushterya ti
    Kogato v skuta ti zaradi muzh zaplache
*/

#include <algorithm>
#include <iostream>
#include <numeric>
#include <cassert>
#include <vector>
#include <queue>

typedef long long llong;
const int MAXN = 5e5 + 10;
const int INF = 1e9;

int n;
int a[MAXN];

struct Triplet
{
    llong value;
    int l, r;

    friend bool operator < (const Triplet &a, const Triplet &b)
    {
        return a.value < b.value;
    }
};


struct SegmentTree
{
    struct Node
    {
        Triplet opt0;
        Triplet opt1;
        int max;
        int min;
        int maxIdx;
        int minIdx;

        Node()
        {
            opt0 = {-INF, 0, 0};
            opt1 = {-INF, 0, 0};
            max = -INF;
            min = INF;
        }

        void set(int value, int idx)
        {
            opt0 = {-INF, 0, 0};
            opt1 = {-INF, 0, 0};
            max = min = value;
            maxIdx = minIdx = idx;
        }

        friend Node operator + (const Node &left, const Node &right)
        {
            Node res;
            res.opt0 = std::max(left.opt0, right.opt0);
            res.opt1 = std::max(left.opt1, right.opt1);
            
            if (left.max > right.max)
            {
                res.max = left.max;
                res.maxIdx = left.maxIdx;
            } else
            {
                res.max = right.max;
                res.maxIdx = right.maxIdx;
            }

            if (left.min < right.min)
            {
                res.min = left.min;
                res.minIdx = left.minIdx;
            } else
            {
                res.min = right.min;
                res.minIdx = right.minIdx;
            }
            
            res.opt0 = std::max(res.opt0, {left.max - right.min, left.maxIdx, right.minIdx});
            res.opt1 = std::max(res.opt1, {right.max - left.min, left.minIdx, right.maxIdx});
            return res;
        }
    };

    Node tree[4*MAXN];
    void build(int l, int r, int node)
    {
        if (l == r)
        {
            tree[node].set(a[l], l);
            return;
        }

        int mid = l + r >> 1;
        build(l, mid, 2*node);
        build(mid + 1, r, 2*node + 1);
        tree[node] = tree[2*node] + tree[2*node + 1];
    }

    Node query(int l, int r, int node, int queryL, int queryR)
    {
        if (queryL <= l && r <= queryR)
        {
            return tree[node];
        }

        int mid = l + r >> 1;
        Node res;

        if (queryL <= mid) res = res + query(l, mid, 2*node, queryL, queryR);
        if (mid + 1 <= queryR) res = res + query(mid + 1, r, 2*node + 1, queryL, queryR);
        return res;
    }

    void build()
    {
        build(1, n, 1);
    }

    Triplet query(int l, int r, bool type)
    {
        Node res = query(1, n, 1, l, r);
        if (type == 0) return res.opt0;
        else return res.opt1;
    }
};

struct Range
{
    llong value;
    int l, r;
    int optL, optR;
    bool type;

    friend bool operator < (const Range &a, const Range &b)
    {
        return a.value < b.value;
    }
};

SegmentTree tree;
std::priority_queue <Range> pq;

Triplet findOpt(int l, int r, bool type) // 0 -> +-, 1 -> -+
{
    if (type == 0)
    {
        return tree.query(l, r, 0);
    } else if (type == 1)
    {
        return tree.query(l, r, 1);
    }
    // if (type == 0)
    // {
    //     llong ans = -INF;
    //     int min = a[r];
    //     int minIdx = r;
    //     int idxL, idxR;
    //     for (int i = r - 1 ; i >= l ; --i)
    //     {
    //         if (ans < a[i] - min)
    //         {
    //             ans = a[i] - min;
    //             idxL = i;
    //             idxR = minIdx;
    //         }

    //         if (min > a[i])
    //         {
    //             min = a[i];
    //             minIdx = i;
    //         }
    //     }

    //     return {ans, idxL, idxR};
    // } else
    // {
    //     llong ans = -INF;
    //     int min = a[l];
    //     int minIdx = l;
    //     int idxL, idxR;
    //     for (int i = l + 1 ; i <= r ; ++i)
    //     {
    //         if (ans < a[i] - min)
    //         {
    //             ans = a[i] - min;
    //             idxR = i;
    //             idxL = minIdx;
    //         }

    //         if (min > a[i])
    //         {
    //             min = a[i];
    //             minIdx = i;
    //         }
    //     }

    //     return {ans, idxL, idxR};
    // }
}

void solve()
{
    tree.build();
    Triplet initial = findOpt(1, n, 1);
    pq.push({initial.value, 0, n + 1, initial.l, initial.r, 1});

    int ptr = 1;
    llong ans = 0;
    while (pq.size() && pq.top().value > 0)
    {
        auto [val, borderL, borderR, idxL, idxR, type] = pq.top();
        pq.pop();

        ans += val;
        ptr++;
        std::cout << ans << ' ';
        if (borderL + 2 < idxL)
        {
            Triplet curr = findOpt(borderL + 1, idxL - 1, type);
            pq.push({curr.value, borderL, idxL, curr.l, curr.r, type});
        }

        if (idxL + 2 < idxR)
        {
            Triplet curr = findOpt(idxL + 1, idxR - 1, type ^ 1);
            pq.push({curr.value, idxL, idxR, curr.l, curr.r, type ^ 1});
        }

        if (idxR + 2 < borderR)
        {
            Triplet curr = findOpt(idxR + 1, borderR - 1, type);
            pq.push({curr.value, idxR, borderR, curr.l, curr.r, type});
        }
    }

    while (ptr <= n)
    {
        std::cout << ans << ' ';
        ptr++;
    }

    std::cout << '\n';
}

void input()
{
    std::cin >> n;
    for (int i = 1 ; i <= n ; ++i)
    {
        std::cin >> a[i];
    }
}

void fastIOI()
{
    std::ios_base :: sync_with_stdio(0);
    std::cout.tie(nullptr);
    std::cin.tie(nullptr);
}

int main()
{
    fastIOI();
    input();
    solve();

    return 0;
}