#include "cactus.h"
#include <iostream>
#include <vector>
#include <algorithm>

using namespace std;

const int maxn = 16005;
vector<int> g[maxn];
bool visited[maxn];
int par[maxn];
vector<pair<int, int>> cycles;
bool inCycle[maxn];
int cycleNum[maxn];
int depth[maxn];

void dfs(int node, int p, int d)
{
    visited[node] = true;
    par[node] = p;
    depth[node] = d;

    for (auto to : g[node])
    {
        if (visited[to] == false)
        {
            dfs(to, node, d + 1);
        }
        else if (to != par[node])
        {
            if (depth[to] > depth[node]) cycles.push_back({to, node});
        }
    }
}

vector<bool> toBinary(int x)
{
    vector<bool> result;

    for (int i = 13; i >= 0; --i)
    {
        if ((x & (1 << i)) != 0)
        {
            result.push_back(1);
        }
        else
        {
            result.push_back(0);
        }
    }

    return result;
}

vector<bool> encode(int n, vector<pair<int, int>> edges)
{
    cycles.clear();

    for (int i = 1; i <= n; ++i)
    {
        g[i].clear();
        visited[i] = false;
        inCycle[i] = false;
    }

    for (auto [x, y] : edges)
    {
        g[x + 1].push_back(y + 1);
        g[y + 1].push_back(x + 1);
    }

    dfs(1, -1, 0);
    vector<bool> result;
    int cntCycleNum = 0;

    for (int i = 0; i < cycles.size(); ++i)
    {
        int x = cycles[i].first, y = cycles[i].second;

        vector<int> nodes;
        ++cntCycleNum;

        while (x != par[y])
        {
            nodes.push_back(x);
            cycleNum[x] = cntCycleNum;
            inCycle[x] = true;
            x = par[x];
        }

        vector<bool> cntNodes = toBinary(nodes.size());

        for (auto bit : cntNodes)
        {
            result.push_back(bit);
        }

        for (auto node : nodes)
        {
            vector<bool> curr = toBinary(node);

            for (auto bit : curr)
            {
                result.push_back(bit);
            }
        }
    }

    for (int i = 0; i <= 13; ++i)
    {
        result.push_back(0);
    }

    for (int i = 1; i <= n; ++i)
    {
        for (auto j : g[i])
        {
            if (i > j) continue;
            if (cycleNum[i] == cycleNum[j] && cycleNum[i] != 0 && cycleNum[j] != 0) continue;

            vector<bool> curr = toBinary(i);
            for (auto bit : curr) result.push_back(bit);
            curr = toBinary(j);
            for (auto bit : curr) result.push_back(bit);
        }
    }

    return result;
}

vector<pair<int, int>> decode(int n, vector<bool> data)
{
    vector<pair<int, int>> edges;
    int pos = -1;

    for (int i = 0; i < data.size(); ++i)
    {
        int cnt = 0;
        int pw = (1 << 13);

        for (int j = i; j <= i + 13; ++j)
        {
            cnt += data[j] * pw;
            pw >>= 1;
        }

        if (cnt == 0)
        {
            pos = i + 14;
            break;
        }

        vector<int> nodes;
        int from = i + 14;

        for (int t = 0; t < cnt; ++t)
        {
            pw = (1 << 13);
            int num = 0;

            for (int j = from; j <= from + 13; ++j)
            {
                num += data[j] * pw;
                pw >>= 1;
            }

            nodes.push_back(num);
            from += 14;
        }

        for (int j = 0; j < nodes.size(); ++j)
        {
            int nxt = (j == nodes.size() - 1 ? 0 : j + 1);
            edges.push_back({nodes[j], nodes[nxt]});
        }

        i = from - 1;
    }

    if (pos != -1)
    {
        for (int i = pos; i < data.size(); i += 28)
        {
            int pw = (1 << 13);
            int x = 0, y = 0;

            for (int j = i; j <= i + 13; ++j)
            {
                x += pw * data[j];
                pw >>= 1;
            }

            pw = (1 << 13);

            for (int j = i + 14; j <= i + 27; ++j)
            {
                y += pw * data[j];
                pw >>= 1;
            }

            edges.push_back({x, y});
        }
    }

    for (int i = 0; i < edges.size(); ++i)
    {
        --edges[i].first;
        --edges[i].second;
    }

    return edges;
}
